@@ -20,7 +20,7 @@ from __future__ import absolute_import
 import fixtures
 import mock
 
-from nova_powervm.virt.powervm import driver
+from nova.virt.powervm import driver
 
 from nova.virt import fake
 from pypowervm.tests import test_fixtures as pvm_fx
@@ -60,20 +60,31 @@ class HostCPUStats(fixtures.Fixture):
             fixtures.MockPatch('nova_powervm.virt.powervm.host.HostCPUStats'))
 
 
+class ComprehensiveScrub(fixtures.Fixture):
+    """Mock out the ComprehensiveScrub."""
+
+    def setUp(self):
+        super(ComprehensiveScrub, self).setUp()
+        self.mock_comp_scrub = self.useFixture(
+            fixtures.MockPatch('pypowervm.tasks.storage.ComprehensiveScrub'))
+
+
 class VolumeAdapter(fixtures.Fixture):
     """Mock out the VolumeAdapter."""
 
+    def __init__(self, patch_class):
+        self.patch_class = patch_class
+
     def setUp(self):
         super(VolumeAdapter, self).setUp()
         self.std_vol_adpt_fx = self.useFixture(
-            fixtures.MockPatch('nova_powervm.virt.powervm.volume.vscsi.'
-                               'VscsiVolumeAdapter', __name__='MockVSCSI'))
+            fixtures.MockPatch(self.patch_class, __name__='MockVolumeAdapter'))
         self.std_vol_adpt = self.std_vol_adpt_fx.mock
         # We want to mock out the connection_info individually so it gives
         # back a new mock on every call.  That's because the vol id is
         # used for task names and we can't have duplicates.  Here we have
         # just one mock for simplicity of the vol driver but we need
-        # mulitiple names.
+        # multiple names.
         self.std_vol_adpt.return_value.connection_info.__getitem__\
             .side_effect = mock.MagicMock
         self.drv = self.std_vol_adpt.return_value
@@ -93,15 +104,11 @@ class PowerVMComputeDriver(fixtures.Fixture):
 
         # Mock active vios
         self.get_active_vios = self.useFixture(fixtures.MockPatch(
-            'nova_powervm.virt.powervm.vios.get_active_vioses')).mock
+            'pypowervm.tasks.partition.get_active_vioses')).mock
         self.get_active_vios.return_value = ['mock_vios']
 
-        # Mock get inactive running vioses
-        self.get_inactive_running_vioses = self.useFixture(
-            fixtures.MockPatch(
-                'nova_powervm.virt.powervm.vios.'
-                'get_inactive_running_vioses')).mock
-        self.get_inactive_running_vioses.return_value = []
+        self.useFixture(fixtures.MockPatch(
+            'pypowervm.tasks.partition.validate_vios_ready'))
 
         self.drv.adapter.read.return_value = ms_http
         self.drv.session = self.drv.adapter.session
@@ -113,6 +120,9 @@ class PowerVMComputeDriver(fixtures.Fixture):
         # Set up the mock CPU stats (init_host uses it)
         self.useFixture(HostCPUStats())
 
+        self.scrubber = ComprehensiveScrub()
+        self.useFixture(self.scrubber)
+
         self.drv = driver.PowerVMDriver(fake.FakeVirtAPI())
         self.drv.adapter = self.useFixture(pvm_fx.AdapterFx()).adpt
         self._init_host()
@@ -121,6 +131,10 @@ class PowerVMComputeDriver(fixtures.Fixture):
         disk_adpt_fx = self.useFixture(DiskAdapter())
         self.drv.disk_dvr = disk_adpt_fx.std_disk_adpt
 
+    def cleanUp(self):
+        self.scrubber.mock_comp_scrub.mock.assert_called_once()
+        super(PowerVMComputeDriver, self).cleanUp()
+
 
 class TaskFlow(fixtures.Fixture):
     """Construct a fake TaskFlow.
