@@ -35,8 +35,8 @@ from pypowervm.utils import transaction as pvm_trans
 from pypowervm.utils import uuid as pvm_uuid
 from pypowervm.utils import validation as vldn
 from pypowervm.wrappers import base_partition as pvm_bp
+from pypowervm.wrappers import iocard as pvm_card
 from pypowervm.wrappers import logical_partition as pvm_lpar
-from pypowervm.wrappers import managed_system as pvm_ms
 from pypowervm.wrappers import network as pvm_net
 from pypowervm.wrappers import shared_proc_pool as pvm_spp
 
@@ -262,16 +262,22 @@ class VMBuilder(object):
             pvm_bp.DedicatedSharingMode.SHARE_IDLE_PROCS_ALWAYS
     }
 
-    def __init__(self, host_w, adapter):
+    def __init__(self, host_w, adapter, slot_mgr=None):
         """Initialize the converter.
 
         :param host_w: The host system wrapper.
+        :param adapter: The pypowervm.adapter.Adapter for the PowerVM REST API.
+        :param slot_mgr: NovaSlotManager for setting/saving the maximum number
+                         of virtual slots on the VM.
         """
         self.adapter = adapter
         self.host_w = host_w
-        self.stdz = lpar_bldr.DefaultStandardize(
-            self.host_w, uncapped_weight=CONF.powervm.uncapped_proc_weight,
-            proc_units_factor=CONF.powervm.proc_units_factor)
+        kwargs = dict(uncapped_weight=CONF.powervm.uncapped_proc_weight,
+                      proc_units_factor=CONF.powervm.proc_units_factor)
+        if slot_mgr is not None:
+            # This will already default if not set
+            kwargs['max_slots'] = slot_mgr.build_map.get_max_vslots()
+        self.stdz = lpar_bldr.DefaultStandardize(self.host_w, **kwargs)
 
     def lpar_builder(self, instance, flavor):
         """Returns the pypowervm LPARBuilder for a given Nova flavor.
@@ -404,9 +410,7 @@ class VMBuilder(object):
 
         # Search for the pool with this name
         pool_wraps = pvm_spp.SharedProcPool.search(
-            self.adapter, name=pool_name,
-            parent_type=pvm_ms.System.schema_type,
-            parent_uuid=self.host_w.uuid)
+            self.adapter, name=pool_name, parent=self.host_w)
 
         # Check to make sure there is a pool with the name, and only one pool.
         if len(pool_wraps) > 1:
@@ -452,12 +456,11 @@ def get_lpar_names(adapter):
     return [x.name for x in get_lpars(adapter)]
 
 
-def get_instance_wrapper(adapter, instance, host_uuid, xag=None):
+def get_instance_wrapper(adapter, instance, xag=None):
     """Get the LPAR wrapper for a given Nova instance.
 
     :param adapter: The adapter for the pypowervm API
     :param instance: The nova instance.
-    :param host_uuid: The host UUID
     :param xag: The pypowervm XAG to be used on the read request
     :return: The pypowervm logical_partition wrapper.
     """
@@ -474,12 +477,11 @@ def get_instance_wrapper(adapter, instance, host_uuid, xag=None):
             raise
 
 
-def instance_exists(adapter, instance, host_uuid, log_errors=False):
+def instance_exists(adapter, instance, log_errors=False):
     """Determine if an instance exists on the host.
 
     :param adapter: The adapter for the pypowervm API
     :param instance: The nova instance.
-    :param host_uuid: The host UUID
     :param log_errors: Indicator whether to log REST data after an exception
     :return: boolean, whether the instance exists.
     """
@@ -527,17 +529,19 @@ def get_vm_qp(adapter, lpar_uuid, qprop=None, log_errors=True):
                 pass
             kwds['helpers'] = helpers
         resp = adapter.read(pvm_lpar.LPAR.schema_type, **kwds)
-    except pvm_exc.Error as e:
-        if e.response.status == 404:
+    except pvm_exc.HttpError as e:
+        # 404 error indicates the LPAR has been deleted (or moved to a
+        # different host)
+        if e.response and e.response.status == 404:
             raise exception.InstanceNotFound(instance_id=lpar_uuid)
         else:
-            LOG.exception(e)
             raise
 
     return jsonutils.loads(resp.body)
 
 
-def crt_lpar(adapter, host_wrapper, instance, flavor, nvram=None):
+def crt_lpar(adapter, host_wrapper, instance, flavor, nvram=None,
+             slot_mgr=None):
     """Create an LPAR based on the host based on the instance
 
     :param adapter: The adapter for the pypowervm API
@@ -545,17 +549,21 @@ def crt_lpar(adapter, host_wrapper, instance, flavor, nvram=None):
     :param instance: The nova instance.
     :param flavor: The nova flavor.
     :param nvram: The NVRAM to set on the LPAR.
+    :param slot_mgr: NovaSlotManager to restore/save the maximum number of
+                     virtual slots.  If omitted, the default is used.
     :return: The LPAR response from the API.
     """
     try:
-        lpar_b = VMBuilder(host_wrapper, adapter).lpar_builder(instance,
-                                                               flavor)
+        lpar_b = VMBuilder(
+            host_wrapper, adapter, slot_mgr=slot_mgr).lpar_builder(instance,
+                                                                   flavor)
         pending_lpar_w = lpar_b.build()
         vldn.LPARWrapperValidator(pending_lpar_w, host_wrapper).validate_all()
         if nvram is not None:
             pending_lpar_w.nvram = nvram
-        lpar_w = pending_lpar_w.create(parent_type=pvm_ms.System,
-                                       parent_uuid=host_wrapper.uuid)
+        lpar_w = pending_lpar_w.create(parent=host_wrapper)
+        if slot_mgr is not None:
+            slot_mgr.register_max_vslots(lpar_w.io_config.max_virtual_slots)
         return lpar_w
     except lpar_bldr.LPARBuilderException as e:
         # Raise the BuildAbortException since LPAR failed to build
@@ -582,7 +590,7 @@ def update(adapter, host_wrapper, instance, flavor, entry=None, name=None):
     """
 
     if not entry:
-        entry = get_instance_wrapper(adapter, instance, host_wrapper.uuid)
+        entry = get_instance_wrapper(adapter, instance)
 
     lpar_b = VMBuilder(host_wrapper, adapter).lpar_builder(instance, flavor)
     lpar_b.rebuild(entry)
@@ -594,11 +602,10 @@ def update(adapter, host_wrapper, instance, flavor, entry=None, name=None):
     return entry.update()
 
 
-def rename(adapter, host_uuid, instance, name, entry=None):
+def rename(adapter, instance, name, entry=None):
     """Rename a VM.
 
     :param adapter: The adapter for the pypowervm API
-    :param host_uuid: The host UUID.
     :param instance: The nova instance.
     :param name: The new name.
     :param entry: The instance pvm entry, if available, otherwise it will
@@ -606,7 +613,7 @@ def rename(adapter, host_uuid, instance, name, entry=None):
     :returns: The updated LPAR wrapper.
     """
     if not entry:
-        entry = get_instance_wrapper(adapter, instance, host_uuid)
+        entry = get_instance_wrapper(adapter, instance)
 
     hyp_name = pvm_util.sanitize_partition_name_for_api(name)
 
@@ -644,7 +651,7 @@ def dlt_lpar(adapter, lpar_uuid):
 
 def power_on(adapter, instance, host_uuid, entry=None):
     if entry is None:
-        entry = get_instance_wrapper(adapter, instance, host_uuid)
+        entry = get_instance_wrapper(adapter, instance)
 
     # Get the current state and see if we can start the VM
     if entry.state in POWERVM_STARTABLE_STATE:
@@ -658,7 +665,7 @@ def power_on(adapter, instance, host_uuid, entry=None):
 def power_off(adapter, instance, host_uuid, entry=None, add_parms=None,
               force_immediate=False):
     if entry is None:
-        entry = get_instance_wrapper(adapter, instance, host_uuid)
+        entry = get_instance_wrapper(adapter, instance)
 
     # Get the current state and see if we can stop the VM
     LOG.debug("Powering off request for instance %(inst)s which is in "
@@ -668,12 +675,17 @@ def power_off(adapter, instance, host_uuid, entry=None, add_parms=None,
     if entry.state in POWERVM_STOPABLE_STATE:
         # Now stop the lpar
         try:
+            LOG.debug("Power off executing for instance %(inst)s.",
+                      {'inst': instance.name})
             power.power_off(entry, host_uuid, force_immediate=force_immediate,
                             add_parms=add_parms)
         except Exception as e:
             LOG.exception(e)
             raise exception.InstancePowerOffFailure(reason=six.text_type(e))
         return True
+    else:
+        LOG.debug("Power off not required for instance %(inst)s.",
+                  {'inst': instance.name})
 
     return False
 
@@ -731,19 +743,35 @@ def get_instance(context, pvm_uuid):
     return None
 
 
-def get_cnas(adapter, instance, host_uuid):
-    """Returns the current CNAs on the instance.
+def get_cnas(adapter, instance, **search):
+    """Returns the (possibly filtered) current CNAs on the instance.
 
     The Client Network Adapters are the Ethernet adapters for a VM.
     :param adapter: The pypowervm adapter.
     :param instance: The nova instance.
-    :param host_uuid: The host system UUID.
+    :param search: Keyword arguments for CNA.search.  If omitted, all CNAs are
+                   returned.
     :return The CNA wrappers that represent the ClientNetworkAdapters on the VM
     """
-    cna_resp = adapter.read(pvm_lpar.LPAR.schema_type,
-                            root_id=get_pvm_uuid(instance),
-                            child_type=pvm_net.CNA.schema_type)
-    return pvm_net.CNA.wrap(cna_resp)
+    meth = pvm_net.CNA.search if search else pvm_net.CNA.get
+
+    return meth(adapter, parent_type=pvm_lpar.LPAR,
+                parent_uuid=get_pvm_uuid(instance), **search)
+
+
+def get_vnics(adapter, instance, **search):
+    """Returns the (possibly filtered) current vNICs on the instance.
+
+    :param adapter: The pypowervm adapter.
+    :param instance: The nova instance.
+    :param search: Keyword arguments for VNIC.search.  If omitted, all VNICs
+                   are returned.
+    :return The VNIC wrappers that represent the virtual NICs on the VM.
+    """
+    meth = pvm_card.VNIC.search if search else pvm_card.VNIC.get
+
+    return meth(adapter, parent_type=pvm_lpar.LPAR,
+                parent_uuid=get_pvm_uuid(instance), **search)
 
 
 def norm_mac(mac):
@@ -760,14 +788,13 @@ def norm_mac(mac):
     return ':'.join(mac[i:i + 2] for i in range(0, len(mac), 2))
 
 
-def update_ibmi_settings(adapter, instance, host_uuid, boot_type):
+def update_ibmi_settings(adapter, instance, boot_type):
     """Update settings of IBMi VMs on the instance.
 
     :param adapter: The pypowervm adapter.
     :param instance: The nova instance.
-    :param host_uuid: The host system UUID.
     :param boot_type: The boot connectivity type of the instance.
     """
-    lpar_wrap = get_instance_wrapper(adapter, instance, host_uuid)
+    lpar_wrap = get_instance_wrapper(adapter, instance)
     entry = ibmi.update_ibmi_settings(adapter, lpar_wrap, boot_type)
     entry.update()
